@@ -173,8 +173,8 @@ class HealthCheck(Enum, metaclass=HealthCheckMeta):
 
     filter_too_much = 2
     """Check for when the test is filtering out too many examples, either
-    through use of :func:`~hypothesis.assume()` or |strategy.filter|,
-    or occasionally for Hypothesis internal reasons."""
+    through use of |assume| or |.filter|, or occasionally for Hypothesis
+    internal reasons."""
 
     too_slow = 3
     """Check for when your data generation is extremely slow and likely to hurt
@@ -249,7 +249,8 @@ class duration(datetime.timedelta):
 def is_in_ci() -> bool:
     # GitHub Actions, Travis CI and AppVeyor have "CI"
     # Azure Pipelines has "TF_BUILD"
-    return "CI" in os.environ or "TF_BUILD" in os.environ
+    # GitLab CI has "GITLAB_CI"
+    return "CI" in os.environ or "TF_BUILD" in os.environ or "GITLAB_CI" in os.environ
 
 
 default_variable = DynamicVariable[Optional["settings"]](None)
@@ -549,12 +550,18 @@ class settings(metaclass=settingsMeta):
     @property
     def database(self):
         """
-        An instance of |ExampleDatabase| that will be
-        used to save examples to and load previous examples from. If ``None``,
-        no storage will be used.
+        An instance of |ExampleDatabase| that will be used to save examples to
+        and load previous examples from.
 
-        See the :ref:`database documentation <database>` for a list of built-in
-        example database implementations, and how to define custom implementations.
+        If not set, a |DirectoryBasedExampleDatabase| is created in the current
+        working directory under ``.hypothesis/examples``. If this location is
+        unusable, e.g. due to the lack of read or write permissions, Hypothesis
+        will emit a warning and fall back to an |InMemoryExampleDatabase|.
+
+        If ``None``, no storage will be used.
+
+        See the :ref:`database documentation <database>` for a list of database
+        classes, and how to define custom database classes.
         """
         from hypothesis.database import _db_for_path
 
