@@ -12,8 +12,9 @@ import inspect
 import math
 import random
 from collections import defaultdict
+from collections.abc import Sequence
 from contextlib import contextmanager
-from typing import Any, NoReturn, Union
+from typing import Any, Callable, NoReturn, Optional, Union
 from weakref import WeakKeyDictionary
 
 from hypothesis import Verbosity, settings
@@ -71,12 +72,12 @@ def assume(condition: object) -> bool:
     return True
 
 
-_current_build_context = DynamicVariable(None)
+_current_build_context = DynamicVariable[Optional["BuildContext"]](None)
 
 
 def currently_in_test_context() -> bool:
     """Return ``True`` if the calling code is currently running inside an
-    :func:`@given <hypothesis.given>` or :doc:`stateful <stateful>` test,
+    :func:`@given <hypothesis.given>` or :ref:`stateful <stateful>` test,
     ``False`` otherwise.
 
     This is useful for third-party integrations and assertion helpers which
@@ -126,13 +127,16 @@ def deprecate_random_in_strategy(fmt, *args):
 
 
 class BuildContext:
-    def __init__(self, data, *, is_final=False, close_on_capture=True):
-        assert isinstance(data, ConjectureData)
+    def __init__(
+        self,
+        data: ConjectureData,
+        *,
+        is_final: bool = False,
+    ) -> None:
         self.data = data
-        self.tasks = []
+        self.tasks: list[Callable[[], Any]] = []
         self.is_final = is_final
-        self.close_on_capture = close_on_capture
-        self.close_on_del = False
+
         # Use defaultdict(list) here to handle the possibility of having multiple
         # functions registered for the same object (due to caching, small ints, etc).
         # The printer will discard duplicates which return different representations.
@@ -140,9 +144,17 @@ class BuildContext:
             defaultdict(list)
         )
 
-    def record_call(self, obj, func, args, kwargs):
+    def record_call(
+        self,
+        obj: object,
+        func: object,
+        args: Sequence[object],
+        kwargs: dict[str, object],
+    ) -> None:
         self.known_object_printers[IDKey(obj)].append(
-            lambda obj, p, cycle, *, _func=func: p.maybe_repr_known_object_as_call(
+            # _func=func prevents mypy from inferring lambda type. Would need
+            # paramspec I think - not worth it.
+            lambda obj, p, cycle, *, _func=func: p.maybe_repr_known_object_as_call(  # type: ignore
                 obj, cycle, get_pretty_function_description(_func), args, kwargs
             )
         )
@@ -275,7 +287,7 @@ def target(observation: Union[int, float], *, label: str = "") -> Union[int, flo
     ``target()`` with any label more than once per test case.
 
     .. note::
-        **The more examples you run, the better this technique works.**
+        The more examples you run, the better this technique works.
 
         As a rule of thumb, the targeting effect is noticeable above
         :obj:`max_examples=1000 <hypothesis.settings.max_examples>`,
